@@ -3,7 +3,7 @@
 ### OPEN SOURCE LICENSE - GNU AFFERO PUBLIC LICENSE Version 3.0 #######
 #
 #    Net::FullAuto Demonstration GUI
-#    Copyright (C) 2000-2015  Brian M. Kelly
+#    Copyright (C) 2000-2016  Brian M. Kelly
 #
 #    This program is free software: you can redistribute it and/or modify
 #    it under the terms of the GNU Affero General Public License as
@@ -27,7 +27,7 @@ use Cwd;
 our $begin_cwd=cwd();
 chdir "$ENV{PAR_TEMP}/inc" if -e "$ENV{PAR_TEMP}/inc";
 
-our $VERSION = '0.01';
+our $VERSION = '0.02';
 
 use 5.010;
 use strict;
@@ -77,8 +77,8 @@ sub SplitterWindow {
    $help->Append( wxID_ABOUT, '' );
 
    $edit->Append( wxID_COPY,  '' );
-   $edit->Append( wxID_FIND,  '' );
-   my $find_again = $edit->Append( -1, "Find Again\tF3" );
+   $edit->Append( wxID_PASTE,  '' );
+   #my $find_again = $edit->Append( -1, "Find Again\tF3" );
 
    $bar->Append( $file, "&File" );
    #$bar->Append( $edit, "&Edit" );
@@ -102,6 +102,7 @@ sub SplitterWindow {
    my $pem_file='';
    my $full_pem='';
    my $ppk_file='';
+   my $tagged;
    my $ip_txt='';
    my $saved_txt='';
    my $cre_file='';
@@ -120,7 +121,7 @@ sub SplitterWindow {
          close FH;
          $ip_txt=~s/^\s*.*?(\d+[.]\d+[.]\d+[.]\d+).*$/$1/s;
       } elsif ($f=~/^saved.txt$/) {
-         open(FH,"<saved.txt");
+         open(FH,"<saved.txt") || warn $!;
          my @lines=<FH>;
          close FH;
          foreach my $line (@lines) {
@@ -130,7 +131,9 @@ sub SplitterWindow {
             $ppk_file=$line if $line=~/ppk$/;
             $full_pem=$line if $line=~/pem$/;
             $cre_file=$line if $line=~/csv$/;
+            $tagged=$line if $line=~/TagFA/;
          }
+         $tagged=~s/TagFA=// if $tagged;
          $pem_file=$full_pem;
          $pem_file=~s/^.*\\(.*)$/$1/;
          last;
@@ -145,21 +148,16 @@ sub SplitterWindow {
    my $media = Wx::MediaCtrl->new($righttop, -1, '', [-1,-1], [-1,-1], 0 );
    my $media2 = Wx::MediaCtrl->new($righttop, -1, '', [-1,-1], [-1,-1], 0 );
    my $media3 = Wx::MediaCtrl->new($righttop, -1, '', [-1,-1], [-1,-1], 0 );
-   my $vid='https://r3---sn-ab5l6nes.googlevideo.com/videoplayback?dur=617.302&ip=69.25.144.254&initcwndbps=4688750&mv=m&ratebypass=yes&source=youtube&ms=au&signature=CCA382E02C2A6E58F582ADF2CD663BEAF974146B.83B792C58FB29BCE980E4C404CC1027300B16B67&requiressl=yes&mm=31&upn=JJpLOAYKmkY&id=o-AHB_PkzC0_bSsJANgu_GNbmo-_uUzMUGWjvRjCwk1Tri&expire=1420788513&key=yt5&mime=video%2Fmp4&ipbits=0&sver=3&sparams=dur%2Cid%2Cinitcwndbps%2Cip%2Cipbits%2Citag%2Cmime%2Cmm%2Cms%2Cmv%2Cratebypass%2Crequiressl%2Csource%2Cupn%2Cexpire&fexp=900718%2C908206%2C916645%2C927622%2C930825%2C932404%2C938628%2C938809%2C9405921%2C941004%2C943917%2C947209%2C947218%2C948124%2C952302%2C952605%2C952901%2C955301%2C957103%2C957105%2C957201%2C959701&mt=1420766869&itag=22';
-   $vid="Learning_Cloud_Computing_With_Amazon_Web_".
-        "Services_What_Is_The_Cloud.mp4";
-   my $vid2='https://r9---sn-vgqsen7z.googlevideo.com/videoplayback?ms=au&source=youtube&dur=382.664&mv=m&mime=video%2Fmp4&ip=75.49.211.167&initcwndbps=376250&fexp=900718%2C908206%2C916645%2C927622%2C930825%2C932404%2C938628%2C938809%2C9405921%2C941004%2C943917%2C947209%2C947218%2C948124%2C952302%2C952605%2C952901%2C955301%2C957103%2C957105%2C957201%2C959701&id=o-AL3m1Tg5lAwFanYTH8Dsp64QmLtSry4iocuReg3GKyJQ&key=yt5&expire=1420791851&mm=31&ipbits=0&mt=1420770225&itag=22&sver=3&upn=x4z3LvYfGgc&sparams=dur%2Cid%2Cinitcwndbps%2Cip%2Cipbits%2Citag%2Cmime%2Cmm%2Cms%2Cmv%2Cratebypass%2Crequiressl%2Csource%2Cupn%2Cexpire&requiressl=yes&ratebypass=yes&signature=B42BF536A709157F30A16B543235B972ECA64EE6.440606815336C90E38117954EBE3A82BDA7EBBF2';
-   $vid2="Introduction_to_Amazon_Elastic_Compute_Cloud_EC2.mp4";
-   $media->LoadURI("THIS.avi");
+   my $vid="https://www.youtube.com/watch?v=dW7fXqYbXS0";
+   my $vid2="Introduction_to_Amazon_Elastic_Compute_Cloud_EC2.mp4";
+   $media->LoadURI("fullauto_demonstration.mp4");
    $media->Show( 1 );
    $media->ShowPlayerControls;
    $righttop->{media}=$media;
    $media2->LoadURI($vid);
-   $media2->Show( 1 );
    $media2->ShowPlayerControls;
    $righttop->{media2}=$media2;
    $media3->LoadURI($vid2);
-   $media3->Show( 1 );
    $media3->ShowPlayerControls;
    $righttop->{media3}=$media3;
 
@@ -167,51 +165,50 @@ sub SplitterWindow {
    EVT_MEDIA_STATECHANGED($righttop, $media2,\&main::on_media2_loaded);
    EVT_MEDIA_STATECHANGED($righttop, $media3,\&main::on_media3_loaded);
 
-if (0) {
-   my $webpanel = Wx::Panel->new($righttop, wxID_ANY);
-
-   my $aws_url='https://portal.aws.amazon.com/gp/aws/developer/'.
-               'registration/index.html';
+   my $webpanel='';my $webpanel2='';my $webpanel3='';
+   $webpanel = Wx::Panel->new($righttop, wxID_ANY);
+   my $aws_url='https://www.youtube.com/embed/gRwa1QoOS7M';
    $webpanel->{defaulturl}=$aws_url;
    my $html = Wx::WebView::New($webpanel, wxID_ANY, $webpanel->{defaulturl});
    $html->{defaulturl}=$aws_url;
    $webpanel->{webview}=$html;
 
-   my $dfbrow  = Wx::Button->new($webpanel, wxID_ANY, 'Go to Default Browser');
-   my $btnurl  = Wx::Button->new($webpanel, wxID_ANY, 'Load URL');
-   my $btnback = Wx::Button->new($webpanel, wxID_ANY, 'Back');
-   my $btnforw = Wx::Button->new($webpanel, wxID_ANY, 'Forward');
-   my $btnhist = Wx::Button->new($webpanel, wxID_ANY, 'History');
-
-   my $buttonsizer = Wx::BoxSizer->new(wxHORIZONTAL);
-   $buttonsizer->Add($dfbrow,  0, wxLEFT|wxRIGHT, 0);
-   $buttonsizer->Add($btnurl,  0, wxLEFT|wxRIGHT, 0);
-   $buttonsizer->Add($btnback, 0, wxLEFT|wxRIGHT, 0);
-   $buttonsizer->Add($btnforw, 0, wxLEFT|wxRIGHT, 0);
-   $buttonsizer->Add($btnhist, 0, wxLEFT|wxRIGHT, 0);
-
-   EVT_BUTTON($webpanel, $dfbrow,  sub { shift->main::GoToDefBrowser( @_ ); });
-   EVT_BUTTON($webpanel, $btnurl,  sub { shift->main::OnBtnURL( @_ ); });
-   EVT_BUTTON($webpanel, $btnback, sub { shift->main::OnBtnBack( @_ ); });
-   EVT_BUTTON($webpanel, $btnforw, sub { shift->main::OnBtnForward( @_ ); });
-   EVT_BUTTON($webpanel, $btnhist, sub { shift->main::OnBtnHistory( @_ ); });
-
    my $msizer = Wx::BoxSizer->new( wxVERTICAL );
    $msizer->Add($html, 1, wxEXPAND|wxALL, 0);
-   $msizer->Add($buttonsizer, 0, wxEXPAND|wxALL, 0);
+   #$msizer->Add($buttonsizer, 0, wxEXPAND|wxALL, 0);
 
    $webpanel->SetSizer( $msizer );
    $webpanel->Layout;
    $webpanel->Refresh;
 
-}
+   $webpanel2 = Wx::Panel->new($righttop, wxID_ANY);
+   $aws_url='https://www.youtube.com/embed/dW7fXqYbXS0';
+   $webpanel2->{defaulturl}=$aws_url;
+   my $html2 = Wx::WebView::New($webpanel2, wxID_ANY, $webpanel2->{defaulturl});
+   $html2->{defaulturl}=$aws_url;
+   $webpanel2->{webview}=$html2;
+   $msizer = Wx::BoxSizer->new( wxVERTICAL );
+   $msizer->Add($html2, 1, wxEXPAND|wxALL, 0);
+   $webpanel2->SetSizer( $msizer );
+   $webpanel2->Layout;
+   $webpanel2->Refresh;
+   $webpanel3 = Wx::Panel->new($righttop, wxID_ANY);
+   $aws_url='https://www.youtube.com/embed/Px7ZPLq4AOU';
+   $webpanel3->{defaulturl}=$aws_url;
+   my $html3 = Wx::WebView::New($webpanel3, wxID_ANY, $webpanel3->{defaulturl});
+   $html3->{defaulturl}=$aws_url;
+   $webpanel3->{webview}=$html3;
+   $msizer = Wx::BoxSizer->new( wxVERTICAL );
+   $msizer->Add($html3, 1, wxEXPAND|wxALL, 0);
+   $webpanel3->SetSizer( $msizer );
+   $webpanel3->Layout;
+   $webpanel3->Refresh;
 
    $self->{media}=$media;
 
-   $righttop->AddPage( $media, " FullAuto Demonstration ", 1 );
-   #$righttop->AddPage( $webpanel, "Amazon Web Services", 0 );
-   $righttop->AddPage( $media2, " What is the CLOUD? ", 0 );
-   $righttop->AddPage( $media3, " Amazon Web Services EC2 ", 0 );
+   $righttop->AddPage( $webpanel, "FullAuto Demonstration", 0 );
+   $righttop->AddPage( $webpanel2, "Amazon Web Services", 0 );
+   $righttop->AddPage( $webpanel3, " Amazon Web Services EC2 ", 0 );
 
    $righttop->Show(1);
 
@@ -246,7 +243,6 @@ if (0) {
    $rightbottom->{underway}=$underway;
    $rightbottom->{demodone}=$demodone;
    $rightbottom->{righttop}=$righttop;
-   #$rightbottom->{html}=$html;
    $rightbottom->DragAcceptFiles(1);
    $rightbottom->SetBackgroundColour(Wx::Colour->new(192,192,192));
    my $fp1 = '';my $fp2='';
@@ -263,7 +259,8 @@ if (0) {
      EVT_FILEPICKER_CHANGED( $rightbottom, $fp1, \&on_change );
      my $csv_f=($full_pem)?$cre_file:'';
      $fp2 = Wx::FilePickerCtrl->new( $rightbottom, -1, $csv_f,
-                "Find and Select AWS Credentials file -> credentials.csv",
+                "Find and Select AWS Credentials file ".
+                "-> credentials.csv",
                 "CSV files (*.csv)|*.csv|All files|*.*",
                 [30, 45], [400,-1],wxFLP_USE_TEXTCTRL);
      $fp2->Show(1);
@@ -273,7 +270,7 @@ if (0) {
 
      my $button1=wxPerl::Button->new(
          $rightbottom,
-         'Launch + Get Key (.pem) File',
+         'Launch + Get Key (.pem)',
          id        => -1,
          position  => [440,15],
          size      => [176,-1],
@@ -287,7 +284,7 @@ if (0) {
 
      my $button2=wxPerl::Button->new(
         $rightbottom,
-        'Get AWS  credentials.csv  File',
+        'Get AWS  credentials.csv',
         id        => -1,
         position  => [440,45],
         size      => [176,-1],
@@ -299,8 +296,8 @@ if (0) {
      $button2->Enable(0) if $full_pem; 
      EVT_BUTTON( $rightbottom, $button2, \&main::OnClick_button2 );
 
-     my $ins=($saved_txt)?'Go To Amazon EC2 Dashboard':
-                          'Get IP Address of Instance';
+     my $ins=($saved_txt)?'Amazon EC2 Dashboard':
+                          'Instance IP Address';
      my $button3=wxPerl::Button->new(
         $rightbottom,
         $ins,
@@ -330,6 +327,7 @@ if (0) {
         name      => 'ipbox',
      );
      $rightbottom->{ipbox}=$ipbox;
+     $self->{ipbox}=$ipbox;
      $ipbox->Show(1);
      $ipbox->Enable(0) if $ip_;
      EVT_TEXT( $rightbottom, $ipbox, \&on_change );
@@ -352,24 +350,29 @@ if (0) {
    my $cleanup_chkbox='';
    my $new_server_chkbox='';
    my $export_putty_chkbox='';
-   my $chk=($pem_file && $ip_txt)?99:169;
+   my $chk=169;
+   $chk=99 if !exists $rightbottom->{firstpaint}
+                          && $pem_file && $ip_txt;
    $cleanup_chkbox=Wx::CheckBox->new($rightbottom,-1,'',
                           [118,$chk],[-1,-1],wxTRANSPARENT_WINDOW);
    $cleanup_chkbox->SetBackgroundColour(Wx::Colour->new(77,77,77));
    $rightbottom->{cucb}=$cleanup_chkbox;
    $self->{cucb}=$cleanup_chkbox;
-   EVT_CHECKBOX( $rightbottom, $cleanup_chkbox, \&main::OnClick_chkbox1 );
+   EVT_CHECKBOX( $rightbottom, $cleanup_chkbox,
+                          \&main::OnClick_chkbox1 );
    $export_putty_chkbox=Wx::CheckBox->new($rightbottom,-1,'',
                           [280,$chk],[-1,-1],wxTRANSPARENT_WINDOW);
    $export_putty_chkbox->SetBackgroundColour(Wx::Colour->new(77,77,77));
    $rightbottom->{epcb}=$export_putty_chkbox;
-   EVT_CHECKBOX( $rightbottom, $export_putty_chkbox, \&main::OnClick_chkbox2 );
-   $new_server_chkbox=Wx::CheckBox->new($rightbottom,-1,'',
-                       [440,$chk],[-1,-1],wxTRANSPARENT_WINDOW);
+   EVT_CHECKBOX( $rightbottom, $export_putty_chkbox,
+                           \&main::OnClick_chkbox2 );
+   $new_server_chkbox=Wx::CheckBox->new($rightbottom,-1,'',[440,$chk],
+                          [-1,-1],wxTRANSPARENT_WINDOW);
    $new_server_chkbox->SetBackgroundColour(Wx::Colour->new(77,77,77));
    $rightbottom->{nscb}=$new_server_chkbox;
    $new_server_chkbox->SetValue(1) unless ($ip_txt || $saved_txt);
-   EVT_CHECKBOX( $rightbottom, $new_server_chkbox, \&main::OnClick_chkbox3 );
+   EVT_CHECKBOX( $rightbottom, $new_server_chkbox,
+                           \&main::OnClick_chkbox3 );
    my $eng = Wx::Bitmap->new("engineroom.jpg",
                          wxBITMAP_TYPE_JPEG );
    my $eht=86;
@@ -397,6 +400,7 @@ if (0) {
    }
    $rightbottom->{ppk_file} = $ppk_file if $ppk_file;
    $rightbottom->{ip_txt} = $ip_txt if $ip_txt;
+   $rightbottom->{tagged} = $tagged if defined $tagged;
    EVT_BUTTON( $rightbottom, $bb, \&fullauto_button);
    EVT_BUTTON( $rightbottom, $er, \&enginerm_button);
    EVT_DROP_FILES( $rightbottom, \&main::on_drop );
@@ -407,9 +411,9 @@ if (0) {
    $rightWindows->SetMinimumPaneSize(5);
 
    if ($pem_file && $ip_txt) {
-      $rightWindows->SplitHorizontally($righttop,$rightbottom,419);
+      $rightWindows->SplitHorizontally($righttop,$rightbottom,414);
    } else {
-      $rightWindows->SplitHorizontally($righttop,$rightbottom,345);
+      $rightWindows->SplitHorizontally($righttop,$rightbottom,340);
    }
    $rightWindows->SetSashGravity(1.0);
    $splitterWindow->SplitVertically($banner,$rightWindows,142);
@@ -418,8 +422,9 @@ if (0) {
    EVT_MENU( $self, wxID_ABOUT, \&on_about );
    EVT_MENU( $self, wxID_EXIT, sub { $self->Close } );
    EVT_MENU( $self, wxID_COPY, \&on_copy );
-   EVT_MENU( $self, wxID_FIND, \&on_find );
-   EVT_MENU( $self, $find_again, \&on_find_again );
+   EVT_MENU( $self, wxID_PASTE, \&on_paste );
+   #EVT_MENU( $self, wxID_FIND, \&on_find );
+   #EVT_MENU( $self, $find_again, \&on_find_again );
 
    $self->SetIcon(Wx::Icon->new("FA.ico",wxBITMAP_TYPE_ICO));
    $self->Show;
@@ -432,13 +437,13 @@ sub on_paint {
     my $dc = Wx::PaintDC->new( $self );
 
     $dc->DrawBitmap( $self->{steel},0,0,0);
-    my $firstpaint=0;my $chk=169;
-    if (!$firstpaint && (exists $self->{ip_txt})) {
+    my $chk=169;
+    my $firstpaint=0;
+    if (!exists $self->{firstpaint} && (exists $self->{ip_txt})) {
        $chk=99;
        $firstpaint=1;
     }
     $self->{firstpaint}=$firstpaint;
-    $chk=(exists $self->{ip_txt})?99:169;
     $dc->DrawText('Launch New FullAuto Server',460,$chk);
     $dc->DrawText('Cleanup on Terminate',139,$chk);
     $dc->DrawText('Export PuTTY',301,$chk);
@@ -446,56 +451,70 @@ sub on_paint {
 }
 
 sub evt_process_exit {
+
     my ($self, $event) = @_;
     $event->Skip(1);
     my $process = $event->GetProcess;
-    #my $line = $event->GetLine;
-    #my @buffers = @{ $process->GetStdOutBuffer };
-    #my @errors = @{ $process->GetStdErrBuffer };
-    #my $exitcode = $process->GetExitCode;
     $process||='';
     $process->Destroy if $process;
-    open (FH,"<putty.log");
-    my @lines=<FH>;
-    close FH;
-    my $out=join '',@lines;
-    my $liferay_url='';
+    my $log_out='';
+    my @lines=();
+    print "Process Exit Event at ".scalar localtime(time())."\n";
+    if (-e "putty.log") {
+       open (FH,"<putty.log");
+       @lines=<FH>;
+       close FH;
+       $log_out=join '',@lines;
+    }
+    my $webapp_url='';
     my $set_shifted_flag=0;
-    my $liferay_flag=0;
+    my $webapp_flag=0;
     if (!exists $self->{shifted_cmds} &&
-          (-1<index $out,'Nothing to do')) {
+          (-1<index $log_out,'Nothing to do') &&
+          (-1==index $log_out,'ACCESS ')) {
       copy "cmds_saved.template", "cmds.txt";
       my $iptext=$self->{ip_txt};
-#print "HERE WE GO and IPTEXT=$iptext\n";
-      my $set_shifted_flag=1;
+      $set_shifted_flag=1;
       my $proc=Wx::Perl::ProcessStream::Process->new(
-            "runputty fullauto.ppk $iptext",'putty',$self);
+            "runputty $self->{ppk_file} $iptext",'putty',$self);
       $self->{shifted_cmds}=$proc;
       $proc->Run;
-    } elsif (!exists $self->{liferay}) {
-      open(PL,"putty.log");
-      while (my $line=<PL>) {
-         if (-1<index $line,'ACCESS LIFERAY PORTAL AT:') {
-            $liferay_url=$line;
-            $liferay_url=~s/^.*ACCESS LIFERAY PORTAL AT:\s+(http.*8080)\s*/$1/;
+    } elsif (!exists $self->{webapp}) {
+      my $hadoop_flag=0;my $count=0;
+      foreach my $line (reverse @lines) {
+         $count++;
+         if ($line=~/ACCESS (?:HADOOP|KALI|OPENLDAP|CATALYST)/) {
             last;
+         } elsif ($line=~/ACCESS.*AT:\s+http:/ && $count<70) {
+            $webapp_url=$line;
+            $webapp_url=~s/^.*ACCESS.*AT:\s+(http.*)\s*$/$1/;
+            last;
+         } elsif ($line=~s/^\s+(http.*)\s*$/$1/ && $count<70) {
+            $webapp_url=$line;
          }
       }
     }
-    if (!exists $self->{liferay} && $liferay_url) {
+    if (!(exists $self->{webapp}) && $webapp_url) {
+      #$self->{donetime}=time()+3600 unless exists $self->{donetime};
       $self->{dashboard}->SetBackgroundColour(Wx::Colour->new(32,161,35));
       $self->{statbm}->SetBitmap($self->{demodone});
-      $liferay_flag=1;
-      $self->{liferay}=1;
-      Wx::LaunchDefaultBrowser($liferay_url,wxBROWSER_NEW_WINDOW);
+      $webapp_flag=1;
+      $self->{webapp}=1;
+      Wx::LaunchDefaultBrowser($webapp_url,wxBROWSER_NEW_WINDOW);
       $self->{bb}->Enable(1);
     } 
-    if ((!exists $self->{shifted_cmds} && !exists $self->{liferay}) ||
-         (!$set_shifted_flag && !$liferay_flag)) {
-      $self->{dashboard}->SetBackgroundColour(Wx::Colour->new(32,161,35));
-      $self->{statbm}->SetBitmap($self->{presshere});
-      $self->{bb}->Enable(1);
+    if ((!(exists $self->{shifted_cmds}) && !(exists $self->{webapp})) ||
+         (!($set_shifted_flag) && !($webapp_flag))) {
+      #if (!$self->{donetime} || $self->{donetime}<time()) {
+         $self->{dashboard}->SetBackgroundColour(Wx::Colour->new(32,161,35));
+         $self->{statbm}->SetBitmap($self->{presshere});
+         $self->{bb}->Enable(1);
+         $self->{key}->Enable(1);
+         $self->{credentials}->Enable(1);
+         $self->{ipbox}->Enable(1) unless -e "putty.log";
+      #}
     }
+
 }
 
 sub fullauto_button {
@@ -503,6 +522,7 @@ sub fullauto_button {
    my ($self, $event) =@_;
    # http://proton-ce.sourceforge.net/rc/wxwidgets \
    # /docs/html/wx/wx_processfunctions.html
+   tag_for_removal($self,$event) unless exists $self->{tagged};
    $self->{key}->Enable(0);
    $self->{credentials}->Enable(0);
    $self->{ipbox}->Enable(0);
@@ -512,6 +532,8 @@ sub fullauto_button {
    $self->{statbm}->SetBitmap($self->{underway});
    $self->{nscb}->SetValue(0);
    $self->{bb}->Enable(0);
+   delete $self->{webapp} if exists $self->{webapp};
+   delete $self->{shifted_cmds} if exists $self->{shifted_cmds};
    my $key=$self->{fp1}->GetPath();
    $key=~s/^.*\/(.*)$/$1/;
    my $keybasename=$key;
@@ -528,19 +550,30 @@ sub fullauto_button {
    unless (-e "putty.log") {
       Wx::ExecuteCommand("puttycfg $iptext",wxEXEC_SYNC)
    }
+   EVT_WXP_PROCESS_STREAM_EXIT($self, \&evt_process_exit);
    unless ((-e "saved.txt") || (-e "ip.txt")) {
       unless (-e "$keybasename.ppk") {
-         Wx::ExecuteCommand("puttykey $key",wxEXEC_SYNC);
+         Wx::ExecuteCommand("puttykey $keybasename $path",wxEXEC_SYNC);
       }
       if (exists $ENV{PAR_TEMP} && (-e "$ENV{PAR_TEMP}/inc")) {
-print "THIS=$path/fullauto.ppk\n";
-         copy "$path/fullauto.ppk", "$ENV{PAR_TEMP}/inc";
+         copy "$path/$keybasename.ppk", "$ENV{PAR_TEMP}/inc";
       } else {
-         copy "$path/fullauto.ppk", cwd();
+         copy "$path/$keybasename.ppk", cwd();
       }
-      unlink "$path/fullauto.ppk";
-      Wx::ExecuteCommand("puttyyes $keybasename.ppk ec2-user $i",
+      unlink "$path/$keybasename.ppk";
+      my ($rcode,$stdout,$stderr)=('','','');
+      ($rcode,$stdout,$stderr)=Wx::ExecuteStdoutStderr(
+         "puttyyes $keybasename.ppk ec2-user $i",
          wxEXEC_SYNC);
+      if ($rcode) {
+         my $title="FullAuto Demo Fatal Error";
+         my $message="Fatal Error: $stderr";
+         my $dialog=Wx::MessageDialog->new(
+            $self,$message,$title,wxOK|wxICON_EXCLAMATION);
+         $dialog->SetOKLabels("FullAuto Setup will Exit");
+         my $choice=$dialog->ShowModal(); 
+         return;
+      }
       $self->{ppk_file}="$keybasename.ppk";
       my $c="pscp.exe -batch -v -i $keybasename.ppk ".
             "MyConfig.pm ec2-user\@$i:/home/ec2-user";
@@ -557,22 +590,11 @@ print "THIS=$path/fullauto.ppk\n";
    } else {
       copy "cmds_saved.template", "cmds.txt";
    }
-   #unless (exists $self->{secret_access_key}) { 
-   #   open(FH,"<credentials.csv");
-   #   my @creds=<FH>;
-   #   close FH;
-   #   my $id='';my $aki='';my $sac='';
-   #   chomp $creds[1];
-   #   ($id,$aki,$sac)=split ',',$creds[1];
-   #   $self->{access_key_id}=$aki;
-   #   $self->{secret_access_key}=$sac;
-   #}
    open(FH,"+<cmds.txt");
    my $out='';my $k=$keybasename.'.pem';
    while(my $line=<FH>) {
-      #$line=~s/_k_/$self->{access_key_id}/eg;
-      #$line=~s/_s_/$self->{secret_access_key}/eg;
       $line=~s/_p_/$k/eg;
+      $line=~s/_c_/$self->{tagged}/eg;
       $out.=$line;
    }
    seek(FH,0,0);
@@ -584,10 +606,11 @@ print "THIS=$path/fullauto.ppk\n";
    print FH $self->{ppk_file}."\n";
    print FH $self->{fp1}->GetPath()."\n";
    print FH $self->{fp2}->GetPath()."\n";
+   print FH "TagFA=".$self->{tagged}."\n";
    close FH;
-   EVT_WXP_PROCESS_STREAM_EXIT($self, \&evt_process_exit);
+print "GOING TO RUNPUTTY\n";
    my $proc1=Wx::Perl::ProcessStream::Process->new(
-         "runputty fullauto.ppk $iptext",'putty',$self);
+         "runputty $self->{ppk_file} $iptext",'putty',$self);
    $proc1->Run;
 
 }
@@ -596,7 +619,6 @@ sub enginerm_button {
 
    my ($self, $event) =@_;
    my $cwd=cwd();
-print "CWD=$cwd\n";
    Wx::Shell("$cwd/engine.exe"); 
 
 }
@@ -612,22 +634,12 @@ sub on_change {
       if (exists $ENV{PAR_TEMP} && (-e "$ENV{PAR_TEMP}/inc")) {
          copy $cred, "$ENV{PAR_TEMP}/inc";
          copy $key,  "$ENV{PAR_TEMP}/inc";
-print "PAR=$ENV{PAR_TEMP}/inc\n";
       } else {
          copy $cred, cwd();
-         #open(FH,"<credentials.csv");
-         #my @creds=<FH>;
-         #close FH;
-         #my $id='';my $aki='';my $sac='';
-         #chomp $creds[1];
-         #($id,$aki,$sac)=split ',',$creds[1];
-         #$self->{access_key_id}=$aki;
-         #$self->{secret_access_key}=$sac;
-#print "SAC=$self->{secret_access_key}\n";
          copy $key, cwd();
       }
       $self->{statbm}->SetBitmap($self->{presshere});
-      $self->{dashboard}->SetLabel('Go To Amazon EC2 Dashboard');
+      $self->{dashboard}->SetLabel('Amazon EC2 Dashboard');
       $self->{dashboard}->SetBackgroundColour(Wx::Colour->new(32,161,35));
       $self->{bb}->SetBitmap($self->{bmp});
       $self->{bb}->Enable(1);
@@ -637,6 +649,40 @@ print "PAR=$ENV{PAR_TEMP}/inc\n";
    }
 }
 
+sub tag_for_removal {
+
+   my ($self, $event) =@_;
+   my $title='Tag FullAuto Server for Later Removal?';
+   my $message='You have just provided an IP Address for a server '.
+               'you have manually launched in the Amazon EC2 Cloud. '.
+               'On this server, FullAuto will install itself, and then '.
+               'proceed with the Demonstration where FullAuto will stand '.
+               'up other servers, and exhibit the completely automated '.
+               'installation and startup of complex software/application '.
+               'architecture spanning multiple hosts. If you click the '.
+               'button \'Tag for Removal\', FullAuto will terminate this '.
+               'server along with all servers launched for the Demonstation '.
+               'when you click the checkbox \'Cleanup on Terminate\' at the '.
+               "bottom of the FullAuto Demonstration interface.\n\nWould you ".
+               "like to tag this server for later termination?\n\n(If you ".
+               'are new to the Cloud, and only wish to view the FullAuto '.
+               'Demonstration, it is recommended you choose \'Tag for '.
+               'Removal\', and later check the box \'Cleanup on Terminate\' '.
+               'when you are done with the Demonstration, so that you don\'t '.
+               'incur any unnecessary Amazon fees.)';
+   my $dialog=Wx::MessageDialog->new(
+      $self,$message,$title,wxOK|wxCANCEL|wxICON_EXCLAMATION);
+   $dialog->SetOKCancelLabels('Tag for Removal','Do NOT Tag');
+   my $choice=$dialog->ShowModal();
+   $choice||=0;
+   if ($choice==5101) {
+      $self->{tagged}=0;
+   } else {
+      $self->{tagged}=1;
+   }
+
+}
+
 sub OnClick_chkbox1 {
 
    my ($self, $event) =@_;
@@ -774,9 +820,6 @@ sub OnClick_button1 {
    my $url='https://portal.aws.amazon.com/gp/aws/developer/'.
            'registration/index.html';
    Wx::LaunchDefaultBrowser($url,wxBROWSER_NEW_WINDOW);
-   #$self->{righttop}->ChangeSelection(1);
-   #$self->{html}->LoadURL($self->{html}->{defaulturl});
-   #print "BUTTON ONE WAS PRESSED\n";
 
 }
 
@@ -785,9 +828,6 @@ sub OnClick_button2 {
    my ($self, $event) =@_;
    my $url='https://console.aws.amazon.com/iam/#users';
    Wx::LaunchDefaultBrowser($url,wxBROWSER_NEW_WINDOW);
-   #$self->{righttop}->ChangeSelection(1);
-   #$self->{html}->LoadURL('https://console.aws.amazon.com/iam/#users');
-   #print "BUTTON TWO WAS PRESSED\n";
 
 }
 
@@ -796,9 +836,6 @@ sub OnClick_button3 {
    my ($self, $event) =@_;
    my $url='https://console.aws.amazon.com/ec2';
    Wx::LaunchDefaultBrowser($url,wxBROWSER_NEW_WINDOW);
-   #$self->{righttop}->ChangeSelection(1);
-   #$self->{html}->LoadURL('https://console.aws.amazon.com/ec2');
-   #print "BUTTON THREE WAS PRESSED\n";
 
 }
 
@@ -969,21 +1006,155 @@ sub on_close {
    my( $self, $event ) = @_;
    chdir $self->{begin_cwd};
    $event->Skip;
+   my $cwd=cwd();
    if ($self->{cucb}->IsChecked()) {
-print "PAR_TEMP=$ENV{PAR_TEMP}\n" if exists $ENV{PAR_TEMP};
       if (exists $ENV{PAR_TEMP}) {
-         my $pardir=$ENV{PAR_TEMP};
+         chdir "$ENV{PAR_TEMP}\\inc";
+         copy "cmds_cleanup.template", "cmds.txt";
+         my $ppk_file='';my $saved_txt;my $full_pem='';
+         my $cre_file='';my $tagged='';
+         opendir(PH,'.');
+         while (my $f=readdir(PH)) {
+            next if $f eq '.';
+            next if $f eq '..';
+            if ($f=~/\.ppk$/) {
+               $ppk_file=$f;
+            } elsif ($f=~/^saved.txt$/) {
+               open(FH,"<saved.txt") || warn $!;
+               my @lines=<FH>;
+               close FH;
+               foreach my $line (@lines) {
+                  chomp $line;
+                  $saved_txt=$line if
+                     $line=~s/^\s*.*?(\d+[.]\d+[.]\d+[.]\d+).*$/$1/s;
+                  $ppk_file=$line if $line=~/ppk$/;
+                  $full_pem=$line if $line=~/pem$/;
+                  $cre_file=$line if $line=~/csv$/;
+                  $tagged=$line if $line=~/TagFA/;
+               }
+               $tagged=~s/TagFA=// if $tagged;
+            }
+         }
+         my $i=$self->{ipbox}->GetLineText(0);
+         my $pem=$full_pem;
+         $pem=~s/^.*\\(.*)/$1/;
+         if (-e "cmds.txt") {
+            open(FH,"+<cmds.txt") || warn $!;
+            my $out='';
+            while(my $line=<FH>) {
+               $line=~s/_p_/$pem/eg;
+               $line=~s/_c_/$tagged/eg;
+               $out.=$line;
+            }
+            seek(FH,0,0);
+            print FH $out;
+            truncate(FH,tell(FH));
+            close(FH);
+            my $ping=Wx::ExecuteStdout("tcping $i 22",wxEXEC_SYNC);
+            $ping=join '',@{$ping};
+            if ((-1<index $ping,'Port is open') && $ppk_file) {
+               my $cmd="runcleanup $ppk_file $i";
+               Wx::ExecuteCommand($cmd,wxEXEC_SYNC)
+            }
+         }
+         my $pardir=$ENV{PAR_TEMP}||'';
          $pardir=~s/^(.*)\\.*$/$1/;
+         copy "saved.txt", "saved.txt.bak";
+         unlink "saved.txt";
+         copy "putty.log", "putty.log.bak";
+         chdir $cwd;
          open(FH,">clean_fullauto_demo.bat");
          print FH "ping 127.0.0.1 -n 1 -w 2000 > nul\n";
+         print FH "del /S /Q $pardir\\inc\\putty.log\n";
+         print FH "rmdir /S /Q $pardir\n";
+         print FH "ping 127.0.0.1 -n 1 -w 2000 > nul\n";
+         print FH "del /S /Q $pardir\\inc\\putty.log\n";
          print FH "rmdir /S /Q $pardir\n";
          print FH 'start /b "" cmd /c del "%~f0"&exit /b';
          close FH;
-         my @arg=('clean_fullauto_demo.bat');
+         my $sessions=
+               'HKEY_CURRENT_USER\\Software\\SimonTatham\\PuTTY\\Sessions';
+         my $sshhkeys=
+               'HKEY_CURRENT_USER\\Software\\SimonTatham\\PuTTY\\SshHostKeys';
+         my $jumplist=
+               'HKEY_CURRENT_USER\\Software\\SimonTatham\\PuTTY\\Jumplist';
+         my @sessions=();my $ses='';
+         open(FH,"reg query $sessions|");
+         while (my $line=<FH>) {
+            next if $line=~/^\s*$/;
+            if (-1<index $line, 'FullAuto') {
+               chomp($line);
+               my $key=$line;
+               $key=~s/^.*\\(.*)\s*$/$1/s;
+               push @sessions, $key;
+               open (DH,"reg delete $line /f 2>&1|");
+               while (my $line=<DH>) {
+                  print $line;
+               }
+               close DH;
+            }
+         }
+         close FH;
+         foreach my $session (@sessions) {
+            my $ip=$session;
+            $ip=~s/^.*_(.*)$/$1/;
+            chomp($ip);
+            open(FH,"reg query $sshhkeys|");
+            while (my $line=<FH>) {
+               next if $line=~/^\s*$/;
+               if (-1<index $line, $ip) {
+                  my $key=$line;
+                  $line=~s/^\s+(.*?)\s+.*$/$1/;
+                  open (DH,"reg delete $sshhkeys /f /v \"$line\" 2>&1|");
+                  while (my $line=<DH>) {
+                     print $line;
+                  }
+                  close DH;
+                  last;
+               }
+            }
+            open(FH,"reg query $jumplist|");
+            while (my $line=<FH>) {
+               next if $line=~/^\s*$/;
+               if ($line=~/FullAuto/) {
+                  my @rsess=split " |.0", $line;
+                  foreach my $rs (@rsess) {
+                     next if $rs=~/^\s*$/;
+                     next if -1<index $rs,'Recent';
+                     next if -1<index $rs,'sessions';
+                     next if -1<index $rs,'REG_MULTI_SZ';
+                     next if -1<index $rs,'FullAuto';
+                     $ses.="$rs ";
+                  }
+                  $ses=~s/\s*$//;
+                  open (DH,
+                     "reg delete $jumplist /f /v \"Recent sessions\" 2>&1|");
+                  while (my $line=<DH>) {
+                     print $line;
+                  }
+                  close DH;
+                  if ($ses) {
+                     my $c="reg add $jumplist /f /v \"Recent sessions\" /t ".
+                           "REG_MULTI_SZ /d \"$ses\"";
+                     open (DH,"$c 2>&1|");
+                     while (my $line=<DH>) {
+                        print $line;
+                     }
+                     close DH;
+                  }
+                  last;
+               }
+            }
+         }
+         my @arg=("clean_fullauto_demo.bat");
          exec @arg;
       }
+      copy "saved.txt", "saved.txt.bak";
+      unlink "saved.txt";
+      copy "putty.log", "putty.log.bak";
+      unlink "putty.log";
    }
-   
+
 }
 
 sub on_about {
@@ -997,7 +1168,7 @@ sub on_about {
    $info->SetVersion( '0.01' );
    $info->SetDescription( 'FullAuto Automates EVERYTHING Demonstration' );
    $info->SetCopyright(
-      "(c) 2000-2015 Brian Kelly <Brian.Kelly\@FullAutoSoftware.net>" );
+      "(c) 2000-2016 Brian Kelly <Brian.Kelly\@FullAutoSoftware.net>" );
    $info->SetWebSite(
       'http://www.FullAutoSoftware.net', 'The FullAuto web site' );
    $info->AddDeveloper( 'Brian Kelly <Brian.Kelly@FullAutoSoftware.net>' );
@@ -1007,6 +1178,34 @@ sub on_about {
 
 }
 
+sub on_paste {
+
+   my( $self ) = @_;
+
+   my $code = $self->{source};
+   my ($from, $to) = $code->GetSelection;
+   my $str = $code->isa( 'Wx::TextCtrl' ) ? $code->GetRange($from, $to)
+                                           : $code->GetTextRange($from, $to);
+   if (wxTheClipboard->Open()) {
+       wxTheClipboard->SetData( Wx::TextDataObject->new($str) );
+       wxTheClipboard->Close();
+   }
+
+   return;
+
+if (wxTheClipboard->Open())
+{
+    if (wxTheClipboard->IsSupported( wxDF_TEXT ))
+    {
+        #wxTextDataObject data;
+        #wxTheClipboard->GetData( data );
+        #wxMessageBox( data.GetText() );
+    }
+    wxTheClipboard->Close();
+}
+
+}
+
 # TODO: disallow copy when not the code is in focus
 # or copy the text from the log window too.
 sub on_copy {
